(function(win, doc) {
  function GridOverlay(targetSel) {
    this.targetEl = doc.querySelector(targetSel);
    if (null === this.targetEl) {
      console.error(`Code Climate layout grid not available: did not find a base element with "${targetSel}" selector`);
      return;
    }

    this.measurements = {
      colCount: 12,
      colWidth: 60,
      gutter: 30,
      baseline: 30
    };
    this.viewState = { cols: false, rows: false };
    this.render();
    this.bindKeys();
  };

  GridOverlay.TOGGLE_KEY = 186; // semicolon

  GridOverlay.prototype = {
    styles: function() {
      var width = (this.measurements.colCount * this.measurements.colWidth)
        + ((this.measurements.colCount - 1) * this.measurements.gutter);
      var height = doc.querySelector("body").clientHeight;

      return `
        .cc-grid-overlay {
          display: none;
          flex-direction: row;
          height: ${height}px;
          left: ${this.targetEl.offsetLeft}px;
          opacity: 0.5;
          position: absolute;
          top: 0;
          width: ${width}px;
          z-index: 999999999;
        }

        .cc-grid-overlay--visible {
          display: flex;
        }

        .cc-grid-overlay > .cc-grid-overlay__rows {
          background: repeating-linear-gradient(
            to bottom,
            transparent 0px, transparent ${this.measurements.baseline - 1}px,
            #ED2A8F ${this.measurements.baseline - 1}px,
            #ED2A8F ${this.measurements.baseline}px
          );
          display: none;
          height: 100%;
          position: absolute;
          top: 0;
          width: 100%;
        }

        .cc-grid-overlay--rows > .cc-grid-overlay__rows {
          display: block;
        }

        .cc-grid-overlay > .cc-grid-overlay__col {
          display: none;
        }

        .cc-grid-overlay--cols > .cc-grid-overlay__col {
          display: block;
          background-color: #D3D3D3;
          height: 100%;
          margin-left:  ${this.measurements.gutter}px;
          width:  ${this.measurements.colWidth}px;
        }

        .cc-grid-overlay > .cc-grid-overlay__col:first-child {
          margin-left: 0;
        }
      `;
    },

    bindKeys: function() {
      doc.addEventListener("keydown", function(ev) {
        if (!ev.ctrlKey) {
          return;
        }
        if (ev.keyCode === GridOverlay.TOGGLE_KEY) {
          if (ev.shiftKey) {
            this.viewState.rows = !this.viewState.rows;
          } else {
            this.viewState.cols = !this.viewState.cols;
          }
          this.setGridState();
        }
      }.bind(this));
    },

    // To live-adjust grid dimensions, you can edit `this.measurements` and
    // call this method.
    render: function() {
      if (this.overlayEl) {
        this.overlayEl.parentElement.removeChild(this.overlayEl);
        this.styleEl.parentElement.removeChild(this.styleEl);
      }

      this.overlayEl = doc.createElement("div");
      this.overlayEl.className = "cc-grid-overlay";
      this.styleEl = doc.createElement("style");
      this.styleEl.id = "cc-grid-styles";
      this.styleEl.innerHTML = this.styles();

      for(var i = 0; i < this.measurements.colCount; i++) {
        this.overlayEl.innerHTML += "<div class=cc-grid-overlay__col />";
      }
      this.overlayEl.innerHTML += "<div class=cc-grid-overlay__rows />";

      var head = doc.querySelector("head");
      var body = doc.querySelector("body");
      head.appendChild(this.styleEl);
      body.appendChild(this.overlayEl);
      this.setGridState();
    },

    toggleOverlayClass: function(predicate, cssClass) {
      if (predicate) {
        this.overlayEl.classList.add(cssClass);
      } else {
        this.overlayEl.classList.remove(cssClass);
      }
    },

    setGridState: function() {
      if (!(this.viewState.cols || this.viewState.rows)) {
        this.overlayEl.classList.remove("cc-grid-overlay--visible");
      } else {
        this.overlayEl.classList.add("cc-grid-overlay--visible");

        this.toggleOverlayClass(this.viewState.cols, "cc-grid-overlay--cols");
        this.toggleOverlayClass(this.viewState.rows, "cc-grid-overlay--rows");
      }
    }
  };

  win.addEventListener("load", function() {
    win.ccDevGridOverlay = new GridOverlay(".container");
    console.group();
    console.log("%cCode Climate layout grid:", "font-weight: bold; color: #3ac189;");
    console.log("  - press `ctrl + ;` to " + "toggle columns");
    console.log("  - press `ctrl + shift + ;` to toggle rows.");
    console.groupEnd();
  });
})(window, document);
